"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from argparse import ArgumentParser
import base64
import datetime
import logging
import os
import re
import unicodedata
import json
import subprocess
import src.Retrospective.ChromeDriver as ChromeDriver
from string import Template

from dateutil.relativedelta import relativedelta

from src.Retrospective.GoogleNewsDriver import GoogleNewsDriver


logger = logging.getLogger(__name__)


class Retrospective:
    def __init__(self):
        self.googleNewsDriver = GoogleNewsDriver()

    def collect_google_news(self, query, period, window, span):
        """Collect Google News search results over a time span.

        Params:
        period - the time period in which separate queries will take place, in months
        window - the time window in which individual queries take place, in weeks
        span - the time span in which the total search takes place, in years
        """
        # Initialize date and results
        current_date = datetime.date.today()
        results = []

        # Check if current search date is within the specified time span
        search_date = current_date
        span_date = current_date - relativedelta(years=+span)
        while search_date >= span_date:

            # Calculate beginning of search window
            max_date = search_date
            min_date = max_date - datetime.timedelta(days=window)

            # Define the URL, then get and parse the page
            print(query)
            print(min_date.strftime("%m/%d/%Y"))
            print(max_date.strftime("%m/%d/%Y"))
            news_url = self.googleNewsDriver.get_google_news_url(
                query,
                min_date=min_date,
                max_date=max_date,
                sorted_by_date=True,
                page_number=1,
            )
            page_source = self.googleNewsDriver.get_search_result_page_source(news_url)
            try:
                result = self.googleNewsDriver.parse_search_result_page_source(page_source)[
                    0
                ]
                results.append(result)
            except:
                print(f"Could not get query: {news_url}")

            # Set the search date for an earlier time span
            search_date = search_date - relativedelta(months=+period)

        return results

    def write_posts(self, repo_path, content_path, image_path, template_path, results):
        """Write GoHugo files for blog posts.

        Params:
        repo_path - path for static site repo
        content_path - absolute path containing hugo list
        image_path - absolute path containing hugo images
        results - collected Google News search results
        """

        content_path = repo_path + content_path
        image_path = repo_path + image_path

        with open(template_path, 'r') as f:
            template = Template(f.read())

        for result in results:

            # Strip new line from title
            title = result["title"].replace('\n', '')

            # Create slugified version of title for use in blog dir
            base_name = self.slugify(title)
            tagged_base_name = f"retrospective-{base_name}"

            # Create absolute blog path for creation of blog dir and files
            blog_path = f"{content_path}/{tagged_base_name}"

            # Create directory if it does not exist
            if not os.path.isdir(blog_path):
                os.makedirs(blog_path, exist_ok=False)
            # Else continue to avoid creating duplicate articles
            else:
                continue

            # Write the image to a file, if image exists
            if result["image"]:
                fields = result["image"].split(",")
                image_extn = fields[0].split("/")[1].split(";")[0]
                image_data = base64.b64decode(fields[1])
                image_name = f"{base_name}.{image_extn}"
                with open(f"{image_path}/{image_name}", "wb") as f:
                    f.write(image_data)
            else:
                image_name = "placeholder.png"

            # Create dictionary for template
            d = {
                'title': title,
                'date': result["date_time"].strftime("%Y-%m-%d"),
                'url': result["link"],
                'image': image_name,
                'body': result["description"],
            }

            text = template.substitute(d)

            # Write the index markdown if image is found
            index_path = f"{blog_path}/index.md"
            with open(index_path, "w+") as f:
                f.writelines(text)

    def output_results(self, repo, content_path, image_path, template, results, config_fname=None):

        if template and config_fname is not None:
            retrospective.add_to_config_file(
                args.search_term, args.period, args.window, args.span, args.repo, config_fname
            )

        if template:
            retrospective.write_posts(
                repo, content_path, image_path, template, results
            )
        else:
            for result in results:
                print(result)

    def slugify(self, value, allow_unicode=False):
        """
        Convert to ASCII if 'allow_unicode' is False. Convert spaces to hyphens.
        Remove characters that aren't alphanumerics, underscores, or hyphens.
        Convert to lowercase. Also strip leading and trailing whitespace.
        Source: https://docs.djangoproject.com/en/3.0/_modules/django/utils/text/#slugify
        """
        value = str(value)
        if allow_unicode:
            value = unicodedata.normalize("NFKC", value)
        else:
            value = (
                unicodedata.normalize("NFKD", value)
                .encode("ascii", "ignore")
                .decode("ascii")
            )
        value = re.sub(r"[^\w\s-]", "", value).strip().lower()
        return re.sub(r"[-\s]+", "-", value)

    def add_to_config_file(self, search_term, period, window, span, repo_path, config_fname):

        search_details = {"period": period, "window": window, "span": span}

        config_path = repo_path + "/" + config_fname

        # Check if file exists
        if os.path.isfile(config_path):
            # Load and resave
            with open(config_path, "r+") as config_file:
                data = json.load(config_file)
                data[search_term] = search_details
                config_file.seek(0)
                json.dump(data, config_file, indent=4)
        else:
            with open(config_path, "w") as config_file:
                data = {}
                data[search_term] = search_details
                config_file.seek(0)
                json.dump(data, config_file, indent=4)

    def populate_from_config_file(self, repo_path, content_path, image_path, template, config_fname):

        config_path = repo_path + "/" + config_fname

        with open(config_path, "r") as config_file:
            searches = json.load(config_file)

            for search in searches:

                results = retrospective.collect_google_news(
                    search,
                    searches[search]["period"],
                    searches[search]["window"],
                    searches[search]["span"],
                )
                retrospective.output_results(repo_path, content_path, image_path, template, results)

    def save(self, tag, repo_dir, ):

        subprocess.run(["rm", "-r", "public"], cwd=repo_dir)
        subprocess.run("hugo", cwd=repo_dir)
        subprocess.run(["hugo", "deploy", "--target=staging"], cwd=repo_dir)
        subprocess.run(["git", "commit", "-am", '"' + tag + '"'], cwd=repo_dir)
        subprocess.run(
            ["git", "tag", "-a", tag, "-m", str(datetime.date.today())], cwd=repo_dir
        )
        subprocess.run(["git", "push", "origin", tag], cwd=repo_dir)

    def load(self, tag, repo_dir):
        subprocess.run(["git", "checkout", tag], cwd=repo_dir)
        subprocess.run(["rm", "-r", "public"], cwd=repo_dir)
        subprocess.run("hugo", cwd=repo_dir)
        subprocess.run(["hugo", "deploy", "--target=staging"], cwd=repo_dir)


def main():
    """
    Conduct a retrospective search to create GoHugo files for blog posts.
    """
    parser = ArgumentParser()
    parser.add_argument(
        "-c",
        "--content-path",
        default="/content/blog",
        help="Relative path containing hugo list built off repo path - [REPO_PATH]/content/blog for static site",
    )
    parser.add_argument(
        "-i",
        "--image-path",
        default="/static/images",
        help="Relative path containing hugo images - [REPO_PATH]/static/images for static site",
    )
    parser.add_argument(
        "-t",
        "--search-term",
        default="bioinformatics",
        help="Term with which to search google news",
    )
    parser.add_argument(
        "-f",
        "--config-fname",
        default="retrospective.json",
        help="Name of retrospective config file in project dir",
    )
    parser.add_argument(
        "-p",
        "--period",
        default=6,
        type=int,
        help="Time period in which separate queries will take place, in months",
    )
    parser.add_argument(
        "-w",
        "--window",
        default=8,
        type=int,
        help="Time window in which individual queries take place, in weeks",
    )
    parser.add_argument(
        "-s",
        "--span",
        default=2,
        type=int,
        help="Time span in which the total search takes place, in years",
    )
    parser.add_argument(
        "--template",
        default=None,
        help="Template file to use in blog. Template can accept title, date, url, image, and body. Value of None will print results",
    )
    parser.add_argument(
        "--show-c",
        action="store_true",
        help="Show copyright",
    )
    parser.add_argument(
        "--show-w",
        action="store_true",
        help="Show warranty",
    )
    parser.add_argument(
        "--from-config", action="store_true", help="populate blog from config file",
    )
    parser.add_argument(
        "--save", help="saves current hugo version with git tag. Overrides other flags",
    )
    parser.add_argument(
        "--load",
        help="loads and deploys git tag to staging area. Overrides other flags",
    )
    #TODO: remove default or make better error message
    parser.add_argument(
        "--repo",
        default="/Users/williamspear/projects/springbok/springbok/springbok-static-site/springbok-site",
        help="Directory location of hugo site git repo",
    )

    args = parser.parse_args()

    print(
        f"""
        Retrospective  Copyright (C) {datetime.datetime.now().year}  Springbok LLC
        This program comes with ABSOLUTELY NO WARRANTY; for details type `Retrospective --show-w'.
        This is free software, and you are welcome to redistribute it
        under certain conditions; type `Retrospective --show-c' for details.
        """
    )
    retrospective = Retrospective()
    ChromeDriver.update()
    if args.show_w:
        print("""
        THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
        APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
        HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
        OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
        THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
        PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
        IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
        ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
        """)
    elif args.show_c:
        print("""
        You may convey verbatim copies of the Program's source code as you
        receive it, in any medium, provided that you conspicuously and
        appropriately publish on each copy an appropriate copyright notice;
        keep intact all notices stating that this License and any
        non-permissive terms added in accord with section 7 apply to the code;
        keep intact all notices of the absence of any warranty; and give all
        recipients a copy of this License along with the Program.

          You may charge any price or no price for each copy that you convey,
        and you may offer support or warranty protection for a fee.
        """)
    elif args.from_config:
        retrospective.populate_from_config_file(
            args.repo, args.content_path, args.image_path, args.template, args.config_fname
        )
    elif args.save:
        retrospective.save(args.save, args.repo)
    elif args.load:
        retrospective.load(args.load, args.repo)
    else:
        results = retrospective.collect_google_news(
            args.search_term, args.period, args.window, args.span
        )
        retrospective.output_results(args.repo, args.content_path, args.image_path, args.template, results, args.config_fname)

if __name__ == "__main__":
    main()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import unittest
import src.Retrospective.ChromeDriver as ChromeDriver

from src.Retrospective.Retrospective import Retrospective


class TestGoogleNewsDriver(unittest.TestCase):
    def setUp(self):
        self.retrospective = Retrospective()
        ChromeDriver.update()
        self.news_url = self.retrospective.googleNewsDriver.get_google_news_url("cisco")
        self.page_source = self.retrospective.googleNewsDriver.get_search_result_page_source(
            self.news_url
        )
        self.results = self.retrospective.googleNewsDriver.parse_search_result_page_source(
            self.page_source
        )

    def test_get_google_news_url(self):
        self.assertEqual(
            self.news_url, "https://www.google.com/search?tbm=nws&q=cisco&tbs=sbd:0", "Should be 'https://www.google.com/search?tbm=nws&q=cisco&tbs=sbd:0'")

    def test_get_search_result_page_source(self):
        self.assertIsNotNone(
            self.page_source, "Should not be None")

    def test_parse_search_result_page_source(self):
        self.assertIsNotNone(
            self.results, "No results returned from parser")
        self.assertTrue(
            "title" in self.results[0])
        self.assertTrue(
            "description" in self.results[0])
        self.assertTrue(
            "link" in self.results[0])
        self.assertTrue(
            "time_ago" in self.results[0])

if __name__ == '__main__':
    unittest.main()


"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import unittest

from ProxyPool import ProxyPool


class ProxyPoolTestCase(unittest.TestCase):
    def setUp(self):
        self.proxyPool = ProxyPool()

    def test_maintain_pool(self):
        self.assertEqual(len(self.proxyPool.instances), 0)
        self.proxyPool.maintain_pool()
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count)

        self.proxyPool.maintain_pool()
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count)

    def test_add_to_pool(self):
        self.proxyPool.maintain_pool()

        self.proxyPool.add_to_pool(1)
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count + 1)

        self.proxyPool.maintain_pool()
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count)

    def test_remove_from_pool(self):
        self.proxyPool.maintain_pool()

        self.proxyPool.remove_from_pool(1)
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count - 1)

        self.proxyPool.maintain_pool()
        self.assertEqual(len(self.proxyPool.instances), self.proxyPool.target_count)

    def test_restart_pool(self):
        self.proxyPool.maintain_pool()

        ip_a = set([i.ip_address for i in self.proxyPool.instances])
        self.proxyPool.restart_pool()
        ip_b = set([i.ip_address for i in self.proxyPool.instances])
        self.assertEqual(len(ip_a.intersection(ip_b)), 0)

    def test_terminate_pool(self):
        self.proxyPool.maintain_pool()

        self.proxyPool.terminate_pool()
        self.assertEqual(len(self.proxyPool.instances), 0)

    def tearDown(self):
        self.proxyPool.terminate_pool()


if __name__ == "__main__":
    unittest.main()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import logging
import random
import time

# TODO: Replace use of boto with boto3
# http://boto.cloudhackers.com/en/latest/
import boto.ec2

# https://boto3.amazonaws.com/v1/documentation/api/latest/index.html
import boto3
import requests
import subprocess

USERNAME = "ubuntu"
KEY_FILENAME = "~/.ssh/tinyproxy-01.pem"

logger = logging.getLogger(__name__)


# TODO: Need a key to make instances unique
class ProxyPool:
    def __init__(
        self,
        region_name="us-east-1",
        image_id="ami-05e8288885ee0c2d5",  # tinyproxy-03
        security_group_id="sg-04ae83dffd0e19255",  # tinyproxy-01
        security_group_rule_ids={
            "ralatsdc": "sgr-0a777fb7a78973c66",
            "spearw": "sgr-0b892e4e6efd4befa",
        },
        target_count=3,
        key_name="tinyproxy-01",
        security_groups=["tinyproxy-01"],
        instance_type="t2.micro",
        sleep_stp=10,
        sleep_max=60,
        **kwargs,
    ):
        self.region_name = region_name
        self.image_id = image_id
        self.security_group_id = security_group_id
        self.security_group_rule_ids = security_group_rule_ids
        self.target_count = target_count
        self.key_name = key_name
        self.security_groups = security_groups
        self.instance_type = instance_type
        self.sleep_stp = sleep_stp
        self.sleep_max = sleep_max
        self.connection = boto.ec2.connect_to_region(self.region_name, **kwargs)
        self.client = boto3.client("ec2")
        self.instances = self._get_instances()

    def maintain_pool(self):
        self.instances = self._get_instances()
        current_count = len(self.instances)
        logger.info("Current count: {0}".format(current_count))
        if current_count < self.target_count:
            self.add_to_pool(self.target_count - current_count)
        elif current_count > self.target_count:
            self.remove_from_pool(current_count - self.target_count)
        self._wait_for_pool(self.target_count)

    def add_to_pool(self, count):
        logger.info("Add count: {0}".format(count))
        self.instances = self._get_instances()
        self.connection.run_instances(
            self.image_id,
            min_count=count,
            max_count=count,
            key_name=self.key_name,
            security_groups=self.security_groups,
            instance_type=self.instance_type,
        )
        self._wait_for_pool(len(self.instances) + count)

    def remove_from_pool(self, count):
        logger.info("Remove count: {0}".format(count))
        self.instances = self._get_instances()
        instance_ids = []
        for i in self.instances:
            instance_ids.append(i.id)
            if len(instance_ids) == count:
                break
        self.connection.stop_instances(instance_ids)
        self._wait_for_pool(len(self.instances) - count)

    def restart_pool(self):
        self.terminate_pool()
        self.maintain_pool()

    def terminate_pool(self):
        self.instances = self._get_instances()
        for i in self.instances:
            logger.info("Terminating instance: {0}".format(i.id))
            i.terminate()
        self._wait_for_pool(0)

    def get_random_proxy_instance(self):
        return random.choice(self.instances)

    def _get_instances(self):
        instances = []
        reservations = self.connection.get_all_reservations()
        for r in reservations:
            for i in r.instances:
                # TODO: Use a key to identify this pool instance
                if (
                    i.image_id == self.image_id
                    and i.instance_type == self.instance_type
                    and i.state == "running"
                ):
                    instances.append(i)
        return instances

    def _wait_for_pool(self, count):
        sleep = 0
        while sleep < self.sleep_max:
            self.instances = self._get_instances()
            if len(self.instances) == count:
                break
            else:
                logger.info("Sleeping {0}".format(self.sleep_stp))
                time.sleep(self.sleep_stp)
                sleep += self.sleep_stp
        self._modify_security_group_rules()

    def _modify_security_group_rules(self):
        # https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/ec2.html#EC2.Client.modify_security_group_rul
        # Get user's github username
        cp = subprocess.run(
            "git config -l | grep remote.origin.url | cut -d ':' -f 2 | cut -d '/' -f 1",
            shell=True,
            capture_output=True,
        )
        username = cp.stdout.decode("utf-8").strip()

        # Get user's public IP address
        ip = requests.get("https://api.ipify.org").text

        # Modify security group rule to permit access by user's IP address
        response = self.client.modify_security_group_rules(
            GroupId=self.security_group_id,
            SecurityGroupRules=[
                {
                    "SecurityGroupRuleId": self.security_group_rule_ids[username],
                    "SecurityGroupRule": {
                        "IpProtocol": "TCP",
                        "FromPort": 8888,
                        "ToPort": 8888,
                        "CidrIpv4": f"{ip}/32",
                        "Description": "ralatsdc",
                    },
                },
            ],
        )
        return response


def main():
    return ProxyPool()


if __name__ == "__main__":
    proxyPool = main()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import unittest

from bs4 import BeautifulSoup as Soup

from ProxyRequests import ProxyRequests


class ProxyRequestTestCase(unittest.TestCase):
    def setUp(self):
        self.proxyRequests = ProxyRequests()

    def test___init__(self):
        self.assertEqual(len(self.proxyRequests.proxy_pool.instances), 3)
        self.assertEqual(len(self.proxyRequests.user_agent.user_agents), 3)

    def test_get_random_user_agent(self):
        url = "https://www.google.com"
        response = self.proxyRequests.get(url)
        soup = Soup(response.content, "html.parser")
        input = soup.find_all("input", attrs={"class": "gLFyf gsfi"})
        self.assertEqual(len(input), 1)

    def tearDown(self):
        self.proxyRequest = None


if __name__ == "__main__":
    unittest.main()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

import unittest

from UserAgent import UserAgent


class UserAgentTestCase(unittest.TestCase):
    def setUp(self):
        self.userAgent = UserAgent()

    def test___init__(self):
        self.assertEqual(len(self.userAgent.user_agents), 0)
        self.userAgent.load_user_agents()
        self.assertEqual(len(self.userAgent.user_agents), 3)

    def test_get_random_user_agent(self):
        self.userAgent.load_user_agents()
        n = 0
        ua_a = self.userAgent.get_random_user_agent()
        for i in range(1000):
            ua_b = self.userAgent.get_random_user_agent()
            if ua_a == ua_b:
                n += 1
        self.assertTrue(330 < n and n < 336)

    def tearDown(self):
        self.userAgent = None


if __name__ == "__main__":
    unittest.main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="GoogleNews",
    version="1.5.5",
    author="Hurin Hu",
    author_email="hurin@live.ca",
    description="Google News search for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Iceloof/GoogleNews",
    packages=setuptools.find_packages(),
    install_requires=['beautifulsoup4','dateparser','python-dateutil'],
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import requests

from ProxyPool import ProxyPool
from UserAgent import UserAgent


class ProxyRequests:
    def __init__(self, target_count=3, max_attempts=3):
        self.target_count = target_count
        self.max_attempts = max_attempts
        self.proxy_pool = ProxyPool()
        self.proxy_pool.maintain_pool()
        self.user_agent = UserAgent()
        self.user_agent.load_user_agents()

    def get(self, url, headers={}):
        attempts = 0
        while attempts < self.max_attempts:
            instance = self.proxy_pool.get_random_proxy_instance()
            proxies = {
                "http": "http://" + instance.ip_address + ":8888",
                "https": "http://" + instance.ip_address + ":8888",
            }
            if not headers:
                headers = {
                    "user-agent": self.user_agent.get_random_user_agent(),
                    # "referer": "https://www.google.com/",
                    # "accept": "*/*",
                    # "accept-encoding": "gzip, deflate, br",
                    # "accept-language": "en-US,en;q=0.9",
                    # "content-length": "0",
                    # "content-type": "text/plain;charset=UTF-8",
                    # "dnt": "1",
                    # "origin": "https://www.google.com",
                    # "sec-fetch-dest": "empty",
                    # "sec-fetch-mode": "no-cors",
                    # "sec-fetch-site": "same-origin",
                }
            response = requests.get(url, proxies=proxies, headers=headers)
            if not response.ok:
                instance.terminate()
                self.proxy_pool.maintain_pool()
            else:
                break
        return response


def main():
    pass


if __name__ == "__main__":
    main()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""

from contextlib import contextmanager
from datetime import datetime, timedelta
import logging
import pprint
import re

from bs4 import BeautifulSoup, ResultSet
from selenium import webdriver
from selenium.webdriver.chrome.options import Options

from src.Retrospective.ProxyPool import ProxyPool
from src.Retrospective.ChromeDriver import DRIVER_HOME


GOOGLE_NEWS_BASE_URL = "https://www.google.com/search?tbm=nws"
DRIVER_PATH = f"{DRIVER_HOME}/chromedriver"


logger = logging.getLogger(__name__)


class GoogleNewsDriver:
    def __init__(self):
        self.proxyPool = ProxyPool()
        self.proxyPool.maintain_pool()

    @contextmanager
    def proxied_driver(self, *args, **kwargs):
        options = Options()
        options.headless = True
        options.add_argument("--window-size=1920,1200")
        instance = self.proxyPool.get_random_proxy_instance()
        options.add_argument(
            "--proxy-server={0}:{1}".format(instance.ip_address, "8888")
        )
        driver = webdriver.Chrome(options=options, executable_path=DRIVER_PATH)
        try:
            yield driver
        finally:
            driver.quit()

    def get_google_news_url(
        self,
        query,
        min_date=None,
        max_date=None,
        sorted_by_date=False,
        page_number=1,
        results_per_page=10,
    ):
        """Google News URLs as of May 31, 2021 modified for simpler
        creation.

        Recent, Sorted by relevance (Default)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:0 (p 1)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:0&start=20 (p 2)

        Recent, Sorted by date, Hide Duplicates
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:1 (p 1)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:1&start=20 (p 2)

        Custom range... Apr 3, 2020 – May 4, 2021, Sorted by relevance
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:0,cdr:1,cd_min:4/3/2020,cd_max:5/4/2021 (pg 1)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:0,cdr:1,cd_min:4/3/2020,cd_max:5/4/2021&start=20 (pg 2)

        Custom range... Apr 3, 2020 – May 4, 2021, Sorted by date (, Hide Duplicates)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:1,cdr:1,cd_min:4/3/2020,cd_max:5/4/2021 (pg 1)
        https://www.google.com/search?tbm=nws&q=apple&tbs=sbd:1,cdr:1,cd_min:4/3/2020,cd_max:5/4/2021&start=20 (pg 2)

        Note that the value of 'start' depends on the Chrome settings of
        the user of this module.
        """
        news_url = GOOGLE_NEWS_BASE_URL + "&q={}".format(query)
        if sorted_by_date:
            news_url += "&tbs=sbd:1"
        else:
            news_url += "&tbs=sbd:0"
        if min_date is not None and max_date is not None:
            news_url += ",cdr:1,cd_min:{0},cd_max:{1}".format(
                min_date.strftime("%m/%d/%Y"), max_date.strftime("%m/%d/%Y"),
            )
        elif min_date is not None or max_date is not None:
            raise Exception("Minimum and maximum date must both be set or None")
        if page_number > 1:
            news_url += "&start={0}".format((page_number - 1) * results_per_page)
        return news_url

    def get_search_result_page_source(self, news_url):
        with self.proxied_driver() as driver:
            driver.get(news_url)
            page_source = driver.page_source
        return page_source

    def parse_search_result_page_source(self, page_source):
        """Selectors are copied from the search results page using
        Chrome Developer Tools by Inspecting Elements and right
        clicking on elements of interest, selecting "Copy >", then
        "Copy selector".
        """
        soup = BeautifulSoup(page_source, "html5lib")
        result_stats = soup.select("#result-stats")
        if result_stats and isinstance(result_stats, ResultSet):
            result_stats = re.search(r"\d+", result_stats[0].text)
            total_count = int(result_stats.group())
            logger.info("Found {0} results".format(total_count))
        else:
            logger.info("No results found")
            return
        g_cards = soup.select("g-card.ftSUBd")
        results = []
        ref_dt = datetime.now()
        if not g_cards:
            return
        for g_card in g_cards:
            # Title
            try:
                # TODO: How do we update these as chrome updates the div classes?
                title = g_card.select("div.mCBkyc.y355M.JQe2Ld.nDgy9d")[
                    0
                ].text
            except Exception as e:
                logger.error("Title not found. Ensure that the div selectors are correct: {0}".format(e))
                title = None
                # Continue, because a lack of a title means that configuration is off
                continue
            # Image
            try:
                image = g_card.select("div.uhHOwf.BYbUcd img")[0].get("src")
            except Exception as e:
                logger.error("Image not found: {0}".format(e))
                image = None
            # Description
            try:
                description = g_card.select("div.GI74Re.nDgy9d")[0].text
            except Exception as e:
                logger.error("Description not found: {0}".format(e))
                description = None
            # Time ago
            try:
                time_ago = g_card.select("div.OSrXXb.ZE0LJd span")[0].text
            except Exception as e:
                logger.error("Time ago not found: {0}".format(e))
                time_ago = None
            # Link
            try:
                link = g_card.select("a[href]")[0].get("href")
            except Exception as e:
                logger.error("Link not found: {0}".format(e))
                link = None
            # Collect results
            date_time = self.time_ago_to_datetime(time_ago, ref_dt=ref_dt) if time_ago else None
            results.append(
                {
                    "title": title,
                    "image": image,
                    "description": description,
                    "time_ago": time_ago,
                    "date_time": date_time,
                    "link": link,
                }
            )
        return results

    def time_ago_to_datetime(self, time_ago, ref_dt=datetime.now()):
        if re.search("ago", time_ago) is not None:
            fields = time_ago.split()
            time_value = int(fields[0])
            time_units = fields[1]
            if re.search("min", time_units):
                ref_dt -= timedelta(minutes=time_value)
            elif re.search("hour", time_units):
                ref_dt -= timedelta(hours=time_value)
            elif re.search("day", time_units):
                ref_dt -= timedelta(days=time_value)
            elif re.search("week", time_units):
                ref_dt -= timedelta(weeks=time_value)
            elif re.search("month", time_units):
                ref_dt -= timedelta(days=time_value * 30)
        else:
            ref_dt = datetime.strptime(time_ago, "%b %d, %Y")
        return ref_dt


if __name__ == "__main__":
    googleNewsDriver = GoogleNewsDriver()
    print(googleNewsDriver.get_google_news_url("apple"))
    # print(googleNewsDriver.get_google_news_url("apple", page_number=2))
    # print(googleNewsDriver.get_google_news_url("apple", sorted_by_date=True))
    # print(googleNewsDriver.get_google_news_url("apple", sorted_by_date=True, page_number=2))
    # print(googleNewsDriver.get_google_news_url("apple", min_date=date.today(), max_date=date.today() + timedelta(days=30)))
    # print(googleNewsDriver.get_google_news_url("apple", min_date=date.today(), max_date=date.today() + timedelta(days=30), page_number=2))
    # print(googleNewsDriver.get_google_news_url("apple", min_date=date.today(), max_date=date.today() + timedelta(days=30), sorted_by_date=True))
    # print(googleNewsDriver.get_google_news_url("apple", min_date=date.today(), max_date=date.today() + timedelta(days=30), sorted_by_date=True, page_number=2))
    news_url = googleNewsDriver.get_google_news_url("cisco")
    page_source = googleNewsDriver.get_search_result_page_source(news_url)
    result = googleNewsDriver.parse_search_result_page_source(page_source)
    pp = pprint.PrettyPrinter(indent=4)
    pp.pprint(result)



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import os
import re
import requests
import wget
import zipfile


DRIVER_HOME = "/usr/local/bin"


def update():
    # Get Chrome application version
    chromereply = os.popen(
        r"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --version"
    ).read()
    chromereply = chromereply.rstrip()
    chromereply = chromereply.lstrip()
    tokens = re.split(r"\s+", chromereply)
    chromefullversion = tokens[2]
    tokens = chromefullversion.split(".")
    chromeversion = tokens[0]

    # Get Chrome driver version
    driverreply = os.popen(r"chromedriver --version").read()
    driverreply = driverreply.rstrip()
    driverreply = driverreply.lstrip()
    tokens = re.split(r"\s+", driverreply)
    driverfullversion = tokens[1]
    tokens = driverfullversion.split(".")
    driverversion = tokens[0]

    # Download and install if driver version does not match application version
    if not driverversion == chromeversion:
        version_number = _get_version(chromeversion)
        download_url = (
            "https://chromedriver.storage.googleapis.com/"
            + version_number
            + "/chromedriver_mac64.zip"
        )
        _download_and_install(download_url)


def _get_version(chromeversion):
    """Get the latest driver for the specifed chrome version
    """
    url = "https://chromedriver.storage.googleapis.com/LATEST_RELEASE_" + str(
        chromeversion
    )
    response = requests.get(url)
    version_number = response.text
    return version_number


def _download_and_install(download_url):
    # Download the zip file using the url built above
    latest_driver_zip = wget.download(download_url)

    # Extract the zip file into the installation directory
    with zipfile.ZipFile(latest_driver_zip, "r") as zip_ref:
        zip_ref.extractall(path=DRIVER_HOME)

    # Delete the zip file downloaded above
    os.remove(latest_driver_zip)


if __name__ == "__main__":
    update()



"""
This project seeks to create a 'retrospective' of Google News top hits.
Copyright (C) Springbok LLC

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""


import os
import random

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


class UserAgent:
    def __init__(
        self,
        data_path=os.path.join(
            BASE_DIR, "Retrospective", "user-agents", "user-agents.txt",
        ),
    ):
        self.data_path = data_path
        self.user_agents = []

    def load_user_agents(self):
        with open(self.data_path, "r") as f:
            self.user_agents = f.readlines()

    def get_random_user_agent(self):
        ua = random.choice(self.user_agents)
        self.user_agents.remove(ua)
        if len(self.user_agents) == 0:
            self.load_user_agents()
        return ua.strip()


def main():
    pass


if __name__ == "__main__":
    main()
